/*
 * MathPlanner 3.1 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */


#include <malloc.h>
#include <string.h>
#include <qfile.h>
#include <stdlib.h>

#include "ConfigReader.h"
#include "Error.h"
#include "Datastore.h"
#include "Extern.h"



DataStorage *config_file_reader::Build()
{
int i,x=0;
char n[8],s[8];

DataStorage *ds=new DataStorage();

for (i=0;i<count;i++) if (data[i].save) {
  sprintf(n,"A%d",x);
  sprintf(s,"B%d",x);
  x++;
  ds->AddString(n,data[i].name);
  ds->AddString(s,data[i].string);
}
return(ds);
}




void config_file_reader::Record(DataStorage *ds)
{

int i,x;
char n[8],s[8];
char *name,*str;

for (i=0;i<256;i++) {
  sprintf(n,"A%d",i);
  sprintf(s,"B%d",i);
  if (ds->Contains(n)) {
    ds->ReadData(n,(void **)&name);
    ds->ReadData(s,(void **)&str);
    for (x=0;x<count;x++) if (strcmp(name,data[x].name)==0) {
             strncpy(data[x].string,str,125);
             break;
             }
  }
  else break;
}

}




bool  config_file_reader::Has(char *name)
{
   int i;
   for(i=0;i<count;i++) {
      if (Match(i,name)) return(true);
   }
   return(false);
}




bool config_file_reader::Match(int Index,char *name)
{
  if (strcmp(name,data[Index].name)==0) return(true);	
	return(false);
}




int config_file_reader::Int(char *name)
{
int i;
for(i=0;i<count;i++) {
   if (Match(i,name)) {
       return(atoi(data[i].string));
   }
}
ErrorReport("ConfigFile: Missing Int",name);
return(0);
}




double config_file_reader::Double(char *name)
{
int i;
for(i=0;i<count;i++) {
   if (Match(i,name)) {
       return(atof(data[i].string));
   }
}
ErrorReport("ConfigFile: Missing Double",name);
return(0.0);
}




QString config_file_reader::String(char *name)
{
int i;
for(i=0;i<count;i++) {
    if (Match(i,name)) {
       return(QString(data[i].string));
   }
}
ErrorReport("ConfigFile: Missing String",name);
return(QString(""));
}




QRect  config_file_reader::Rect(char *a,char *b)
{
   return(QRect(0,0,Int(a),Int(b)));
}




QColor  config_file_reader::Color(char *name)
{
   int i,r=0,g=0,b=0;
   char *co,*tok;
   char se[]=" ,;";

   for(i=0;i<count;i++) {
      if (Match(i,name)) {
      co=strdupa(data[i].string);
      if (co) {
      tok=strtok(co,se); r=atoi(tok);
      tok=strtok(NULL,se); g=atoi(tok);
      tok=strtok(NULL,se); b=atoi(tok);
      }
      return(QColor(r,g,b));
      }
   }

ErrorReport("ConfigFile: Missing Color",name);
return(QColor(0,0,0));
}


// Write Data

void config_file_reader::SetColor(char *name,QColor c)
{
int i;
int r=c.red();
int g=c.green();
int b=c.blue();
char str[64];
sprintf(str,"%d %d %d",r,g,b);
for(i=0;i<count;i++) {
      if (Match(i,name)) {
          strcpy(data[i].string,str);
      }
}
}

void config_file_reader::SetString(char *name,QString c)
{
int i;
for(i=0;i<count;i++) {
      if (Match(i,name)) {
          strcpy(data[i].string,c.latin1());
      }
}
}


void config_file_reader::SetInt(char *name,int c)
{
int i;
char str[64];
sprintf(str,"%d",c);
for(i=0;i<count;i++) {
      if (Match(i,name)) {
          strcpy(data[i].string,str);
      }
}
}





config_file_reader::config_file_reader()
{
  rawdata=NULL;
  data=(cfr_data *)malloc(256*sizeof(cfr_data));
  Load();
}




void config_file_reader::Load()
{
  if (rawdata) free(rawdata);

  QFile file(config_path+"/MPLConfig.txt");
	if (file.exists()) {

	file.open( IO_ReadOnly );
	size=file.size();
	
	rawdata=(char *)malloc(size+16);	
	
	if (!rawdata) ErrorReport("config_file_reader / Creator","Memory error");
	else {
	      file.readBlock(rawdata,size);
        Init();
   }
	file.close();
  return;
	}

  file.setName(install_dir+"/MPLConfig.txt");

  if (file.exists()) {
	file.open( IO_ReadOnly );
	size=file.size();
	
	rawdata=(char *)malloc(size+16);	
	
	if (!rawdata) ErrorReport("config_file_reader / Creator","Memory error");
	else {
	      file.readBlock(rawdata,size);
        Init();
   }
	file.close();
  return;
	}


  ErrorReport("config_file_reader / Creator","Configuration file not found /Home/.MPLConfig.txt\n or /opt/MathPlanner3/MPLConfig.txt");
}


                 

void config_file_reader::Save(QString file_name)
{
  QFile file(file_name);
	file.open( IO_WriteOnly );	
	file.writeBlock(rawdata,size);
	file.close();
}




void config_file_reader::Init()
{
          int i,index,x;
				
          count=0;

          for(i=0;i<size;i++) if (rawdata[i]==';') count++;

          Bug("Reader Init-> Entries"); Bug((long int)count);

          if (count>0 && count<256) {
               index=0;
               for(i=0;i<size;i++) {
                 if (rawdata[i]==10 && rawdata[i+1]!='/' && rawdata[i+1]!=10 && rawdata[i+1]!=32) {

                     if (rawdata[i+1]=='&') data[index].save=true,i++;
                     else data[index].save=false;
                     
                     i++;
                     x=0;
                     while(rawdata[i]!=32 && x<32) data[index].name[x++]=rawdata[i++];
                     data[index].name[x++]=0;

                     while(rawdata[i]==32 || rawdata[i]==9) i++;
                     x=0;
                     while(rawdata[i]!=';' && x<128)
                           data[index].string[x++]=rawdata[i++];
                     data[index].string[x++]=0;
                     i--;

                     Bug(data[index].name);
                     Bug(data[index].string);

                 index++;
                 }
               }
          }
          else ErrorReport("config_file_reader::Init ","Error in config file");
}





void config_file_reader::CreateFile()
{
    int newlen=2*size;
    int i,index=0,x=0,z=0;

    char *newfile=(char *)malloc(newlen);

    memset(newfile,0,newlen);

    for(i=0;i<(size-1);i++) {

        newfile[z++]=rawdata[i];

        if (rawdata[i]==10 && rawdata[i+1]!='/' && rawdata[i+1]!=32 && rawdata[i+1]!=10) {

           if (data[index].save) newfile[z++]='&';
           
           for(x=0;(unsigned int)x<strlen(data[index].name);x++)
               newfile[z++]=data[index].name[x];

           newfile[z++]=32;

           for(x=0;(unsigned int)x<strlen(data[index].string);x++)
               newfile[z++]=data[index].string[x];

           newfile[z++]=';';

           while(rawdata[i]!=';' && i<size) i++;

           index++;
        }



    }

    for(i=0;i<newlen;i++) if (newfile[i]==0) size=i,i=newlen;

    if (rawdata) free(rawdata);
    rawdata=newfile;
    newfile=NULL;

    Save(config_path+"/MPLConfig.txt");
}







config_file_reader::~config_file_reader()
{
    if (rawdata) free(rawdata);
    if (data) free(data);
}

