/*
 * MathPlanner 3.1.3 - Mathematical design tool.
 * Copyright(C) 2004  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */


#ifndef _MATHPLANNER_EXTERN_H
#define _MATHPLANNER_EXTERN_H


#define _MPL_VERSION "3.1.3"
#define _MPL_DATE "Aug 2004"

#define MPL_MF_BW 0
#define MPL_MF_ALPHA 1

#define ARG_TYPE_UNDEFINED 0
#define ARG_TYPE_INTEGER 1
#define ARG_TYPE_FUNCTION 2

#define SUB_NONE  0
#define SUB_LEFT  1
#define SUB_RIGHT 2


#define PRIORITY_NONE	    		0x0000
#define PRIORITY_ASSIGN	   		0x0001  // =
#define PRIORITY_PLUS      		0x0002  // +, -
#define PRIORITY_MULTIBLY  		0x0004  // *, /
#define PRIORITY_MUL_LOW   		0x0008  // 2ln x, 2sin x, 2sqrt
#define PRIORITY_TRIGONO   		0x0010  // sin, cos, log, ln, sqrt.... (-) = negation
#define PRIORITY_MUL_HIGH  		0x0020  // 2x, 2Pi
#define PRIORITY_POWER     		0x0040  // 2^3, 2^(1+2)



#define FONT_SIZE_NORMAL 0
#define FONT_SIZE_SMALL 1
#define FONT_SIZE_EXPONENT_SMALL 2
#define FONT_SIZE_INDEX 3
#define FONT_SIZE_INDEX_SMALL 4



#define FONT_TYPE_NORMAL      0
#define FONT_TYPE_SYMBOL      1
#define FONT_TYPE_INDEX       2
#define FONT_TYPE_OPERATOR    3
#define FONT_TYPE_FUNCTION    4
#define FONT_TYPE_EXPONENT    5


#define MODE_ADD_LEFT 0
#define MODE_ADD_RIGHT 1
#define MODE_INSERT 2
#define MODE_AUTO 3

#define CALL_MODIFY 1
#define CALL_PROTECT_OR_RECONNECT 2
#define CALL_SYNTAXCHK 3
#define CALL_PROTECT 4
#define CALL_UNPROTECT 5

// OPERATORS

#define T_SIN 1
#define T_COS 2
#define T_TAN 3
#define T_COT 4
#define T_ASIN 5
#define T_ACOS 6
#define T_ATAN 7
#define T_ABS 8
#define T_INT 9
#define T_FRAC 10
#define T_INTEG 11
#define T_RAND 12         // Not Ready
#define T_PERMUTATION 13  // Not Ready
#define T_COMBINATION 14  // Not Ready
#define T_FACTORIAL 15
#define T_PAR_A 16
#define T_PAR_B 17
#define T_PAR_C 18
#define T_DIVE 19
#define T_SQRT 20
#define T_POWER 21
#define T_LOG 22
#define T_LN 23
#define T_EXP 24
#define T_E 25
#define T_FUNCTION_DEF 26
#define T_FUNCTION 27
#define T_ACOT 28
#define T_SINH 29
#define T_COSH 30
#define T_TANH 31
#define T_COTH 32
#define T_ASINH 33
#define T_ACOSH 34
#define T_ATANH 35
#define T_ACOTH 36
#define T_SIGMA 37
#define T_BIG_PII 38
#define T_MIN 39
#define T_MAX 40
#define T_IMAGINARY 41
#define T_VECTOR_I 42
#define T_VECTOR_J 43
#define T_VECTOR_K 44
#define T_ARG_Z 45
#define T_MODUL_Z 46
#define T_REZ 47
#define T_IMZ 48
#define T_COMPLEX_A 49
#define T_FUNCTION_DEF_2 50
#define T_FUNCTION_DEF_3 51
#define T_CROSS 52
#define T_LENGTH 53
#define T_CONJUGANT 54
#define T_UNIT 55
#define T_INTEGRAL 56
#define T_LOOP 57
#define T_MATRIX 58



#define OT_BASE_OBJECT 1
#define OT_INTEGER_OBJECT 2
#define OT_VALUE_OBJECT 3
#define OT_TEMP_OBJECT 4
#define OT_SYMBOL_OBJECT 5
#define OT_ASSIGN_OBJECT 6
#define OT_PLUS_OBJECT 7
#define OT_MINUS_OBJECT 8
#define OT_MULTIBLY_OBJECT 9
#define OT_DIVE_OBJECT 10
#define OT_SQRT_OBJECT 11
#define OT_TRIGONO_OBJECT 12
#define OT_PARENTHESIS_OBJECT 13
#define OT_EXPONENT_OBJECT 14
#define OT_FUNCTION_DEFINATION_OBJECT 15
#define OT_FUNCTION_HEADER_OBJECT 16
#define OT_SIGMA_OBJECT 17
#define OT_MINMAX_OBJECT 18
#define OT_INLINE_DIVE_OBJECT 19
#define OT_IMAGINARY_OBJECT 20
#define OT_COMPLEX_OBJECT 21
#define OT_CROSS_OBJECT 22
#define OT_CONJUGANT_OBJECT 23
#define OT_FACTORIAL_OBJECT 24
#define OT_UNIT_OBJECT 25
#define OT_MATRIX_OBJECT 26




#define OMT_STRING_OPERATOR 1
#define OMT_DATA 2
#define OMT_OPERATOR 4


#define DIGIT_MODE_AUTO  0
#define DIGIT_MODE_EXP   1
#define DIGIT_MODE_EXP3  2

#define TO_FRAME        1
#define TO_MAIN         2
#define TO_APPCONTROL   3
#define TO_STRING       4
#define TO_OPERATOR     5
#define TO_MATHCONTROL  6
#define TO_PAPER        7


#define MSG_FONTSIZE    1
#define MSG_STATUS      2
#define MSG_DEG         3
#define MSG_RAD         4
#define MSG_REMOVE      5
#define MSG_UNIT        6
#define MSG_FUNCTION    7
#define MSG_DATA        8
#define MSG_SIGNAL      9
#define MSG_1           10
#define MSG_2           11
#define MSG_3           12
#define MSG_COPY        13
#define MSG_CUT         14
#define MSG_PASTE       15
#define MSG_SELECTALL   16
#define MSG_MATHCOMMAND 17
#define MSG_IMAGEEDIT   18
#define MSG_PASTEPOSITION 19
#define MSG_TEXTFRAME   20
#define MSG_IMAGEFRAME  21
#define MSG_MULTISELECTION 22
#define MSG_RESETNAME   23
#define MSG_CONFIGURATION 24
#define MSG_PIICONSTANT 25



#define ICON(x) AppControl->LoadIcon(x)

#define MESSAGE(x) &PublicMsg; PublicMsg.Clear(); PublicMsg.AddInt("type",x)

#define POSTMESSAGE(x,y)  PublicMsg.Clear(); \
                          PublicMsg.AddInt("type",x); \
                          PostMessage(&PublicMsg,y)

#define ISMSG(x) msg->ReadInt("type")==x
#define SETMSG(x,y) x->AddInt("type",y)

extern class DataStorage PublicMsg;

extern QString image_path;
extern QString home_path;
extern QString install_dir;
extern QString config_path;
extern QString webbrowser;
extern QString doc_dir;
extern QString kde_dir;

#endif

