/*
 * MathPlanner 3.2 - Mathematical design tool.
 * Copyright(C) 2003  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */

#include "MathHeaders.h"
#include <malloc.h>

math_node mpl_neg(math_node a)
{
        if (a.type==MN_REAL)     a.R=-a.R;
				if (a.type==MN_COMPLEX)  a.R=-a.R, a.I=-a.I;
				if (a.type==MN_VECTOR)   a.I=-a.I, a.J=-a.J, a.K=-a.K;
        return(a);
}


math_node mpl_add(math_node a,math_node b)
{
        math_node ans;
        ans.type=MN_NULL;

        if (a.type==MN_REAL) {
               if (b.type==MN_REAL) ans.R=a.R+b.R, ans.type=MN_REAL;
               else if (b.type==MN_COMPLEX) ans.R=a.R+b.R, ans.I=b.I, ans.type=MN_COMPLEX;            
        }
        else
        if (a.type==MN_COMPLEX) {
               if (b.type==MN_REAL) ans.R=a.R+b.R, ans.I=a.I, ans.type=MN_COMPLEX;
               else if (b.type==MN_COMPLEX) ans.R=a.R+b.R, ans.I=a.I+b.I, ans.type=MN_COMPLEX;           
        }
        else 
        if (a.type==MN_VECTOR) {
               if (b.type==MN_VECTOR) ans.I=a.I+b.I, ans.J=a.J+b.J, ans.K=a.K+b.K, ans.type=MN_VECTOR;
        }

        return(ans);
}


math_node mpl_sub(math_node a,math_node b)
{
        math_node ans;
        ans.type=MN_NULL;

        if (a.type==MN_REAL) {
               if (b.type==MN_REAL) ans.R=a.R-b.R, ans.type=MN_REAL;
               else if (b.type==MN_COMPLEX) ans.R=a.R-b.R, ans.I=0-b.I, ans.type=MN_COMPLEX;          
        }
        else
        if (a.type==MN_COMPLEX) {
               if (b.type==MN_REAL) ans.R=a.R-b.R, ans.I=a.I, ans.type=MN_COMPLEX;
               else if (b.type==MN_COMPLEX) ans.R=a.R-b.R, ans.I=a.I-b.I, ans.type=MN_COMPLEX;
        }
        else 
        if (a.type==MN_VECTOR) {
               if (b.type==MN_VECTOR) ans.I=a.I-b.I, ans.J=a.J-b.J, ans.K=a.K-b.K, ans.type=MN_VECTOR;
        }

        return(ans);
}

math_node mpl_mul(math_node a,math_node b)
{
        math_node ans;
        ans.type=MN_NULL;

        if (a.type==MN_REAL) {
               if (b.type==MN_REAL) ans.R=a.R*b.R, ans.type=MN_REAL;
               else if (b.type==MN_COMPLEX) ans.R=a.R*b.R, ans.I=a.R*b.I, ans.type=MN_COMPLEX;
               else if (b.type==MN_VECTOR) ans.I=a.R*b.I, ans.J=a.R*b.J, ans.K=a.R*b.K, ans.type=MN_VECTOR;
        }
        else
        if (a.type==MN_COMPLEX) {
               if (b.type==MN_REAL) ans.R=a.R*b.R, ans.I=a.I*b.R, ans.type=MN_COMPLEX;
               else if (b.type==MN_COMPLEX) ans.R=a.R*b.R-a.I*b.I, ans.I=a.R*b.I+a.I*b.R, ans.type=MN_COMPLEX;
        }
        else 
        if (a.type==MN_VECTOR) {  // Dot product
               if (b.type==MN_VECTOR) ans.R=a.I*b.I+a.J*b.J+a.K*b.K, ans.type=MN_REAL;
               else if (b.type==MN_REAL) ans.I=a.I*b.R, ans.J=a.J*b.R, ans.K=a.K*b.R, ans.type=MN_VECTOR;
        }

        return(ans);
}



math_node mpl_div(math_node a,math_node b)
{
        math_node ans,opz;
        ans.type=MN_NULL;

        if (a.type==MN_REAL) {
               if (b.type==MN_REAL) ans.R=a.R/b.R, ans.type=MN_REAL; // ok
               else if (b.type==MN_COMPLEX) { opz=mpl_opz(b.Complex()); ans.R=a.R*opz.R, ans.I=a.R*opz.I, ans.type=MN_COMPLEX; }
               else if (b.type==MN_VECTOR) ans.I=a.R/b.I, ans.J=a.R/b.J, ans.K=a.R/b.K, ans.type=MN_VECTOR;
        }
        else
        if (a.type==MN_COMPLEX) {
               if (b.type==MN_REAL) ans.R=a.R*(1/b.R), ans.I=a.I*(1/b.R), ans.type=MN_COMPLEX;
               else if (b.type==MN_COMPLEX) { opz=mpl_opz(b.Complex()); ans.R=a.R*opz.R-a.I*opz.I, ans.I=a.R*opz.I+a.I*opz.R, ans.type=MN_COMPLEX; }
        }
        else
        if (a.type==MN_VECTOR) {
               if (b.type==MN_VECTOR) ans.I=a.I/b.I, ans.J=a.J/b.J, ans.K=a.K/b.K, ans.type=MN_VECTOR;
               else if (b.type==MN_REAL) ans.I=a.I/b.R, ans.J=a.J/b.R, ans.K=a.K/b.R, ans.type=MN_VECTOR;
        }

        return(ans);
}





math_node::math_node()
{
Clear();
}


math_node::~math_node()
{

}


math_node::math_node(mpl_real r)
{
real=r;
type=MN_REAL;
}


math_node::math_node(mpl_complex z)
{
complex=z;
type=MN_COMPLEX;
}


math_node::math_node(mpl_rational r)
{
rational=r;
type=MN_RATIONAL;
}


math_node::math_node(mpl_vector v)
{
vector=v;
type=MN_VECTOR;
}





void math_node::Clear()
{
R=I=K=J=0.0;
type=MN_REAL;
}


bool math_node::isNull()
{
if (type==MN_NULL) return(true);
return(false);
}


void math_node::SetReal(mpl_real x)
{
real=x;
type=MN_REAL;
}


void math_node::SetComplex(mpl_complex z)
{
complex=z;
type=MN_COMPLEX;
}


void math_node::SetRational(mpl_rational r)
{
rational=r;
type=MN_RATIONAL;
}


mpl_real math_node::Real()
{
    if (type==MN_RATIONAL) return(R/I);
    return(real);
}


mpl_complex math_node::Complex()
{  
    if (type==MN_REAL) complex.i=0; 
    return(complex);
}


mpl_rational math_node::Rational()
{
    return(rational);
}


mpl_vector math_node::Vector()
{
    return(vector);
}



