/*
 * MathPlanner 3.2 - Mathematical design tool.
 * Copyright(C) 2003  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */


// RATIONAL MATH

#include "MathHeaders.h"
#include <malloc.h>
#include "Error.h"

mpl_real *table_of_numbers=NULL;
mpl_real per_table[30];
int count_of_numbers=0;


mpl_real mpl_rat2real(mpl_rational a)
{
        return(a.a/a.b);
}


mpl_rational mpl_power(mpl_rational a,mpl_real n)
{
        mpl_rational v;
        v.a=pow(a.a,n);
        v.b=pow(a.b,n);
        return(v);
}


mpl_rational mpl_decrade(mpl_rational r)
{
        mpl_rational c;
        int i;
        mpl_real n,up=r.a,lo=r.b;
        if (table_of_numbers) {

        for(i=0;i<count_of_numbers;i++) {
          n=table_of_numbers[i];
          while (fmod(up,n)==0 && fmod(lo,n)==0) {
		        up/=n; lo/=n;
            if (up==1.0 || lo==1.0) i=count_of_numbers;								
		      }
        }
        }
        c.a=up; c.b=lo;
        return(c);
}





bool create_table_of_numbers(int n)
{
        mpl_real x,y;
        int i,k,a,b,d=0;
        bool miss;

        table_of_numbers=(mpl_real *)malloc((n+2)*sizeof(mpl_real));

        if (table_of_numbers) {

        for(a=2,x=2.0;a<n;a++,x+=1.0) { miss=false;
        for(b=2,y=2.0;b<a && miss==false;b++,y+=1.0) {
           if (fmod(x,y)==0.0) miss=true; // break count
        }
        if (!miss && d<n) table_of_numbers[d++]=x;
        }
        count_of_numbers=(d-1);

        }
        else return(false);


        per_table[0]=42336000.0; 
        per_table[1]=2162160.0;
        per_table[2]=1627920.0;
        per_table[3]=1680840.0;
        per_table[4]=2890440.0;
        per_table[5]=4442760.0;
        per_table[6]=6277320.0;
        per_table[7]=1511580.0;
        per_table[8]=3312738.0;
        per_table[9]=4739702.0*13.0;

        if (count_of_numbers>70)
        for(i=0,k=18;i<12;i++,k+=3) {
        per_table[10+i]=60.0*table_of_numbers[k]*table_of_numbers[k+1]*table_of_numbers[k+2];
        }
        else ErrorReport("Mathematics","Table too small");

        return(true);
}




void  delete_table_of_numbers()
{
        if (table_of_numbers) free(table_of_numbers);
        count_of_numbers=0;
}



bool create_root(mpl_real real,mpl_real *in,mpl_real *out)
{
				int i,count;
				mpl_real n;
        if (real<1) return(false);

        if (table_of_numbers && count_of_numbers>0) {
        *in=1.0;
        *out=1.0;
        for(i=0;i<count_of_numbers;i++) {

                n=table_of_numbers[i];
                count=0;

                while(fmod(real,n)==0) {
										real=real/n;
										count++;
								}
                  if (count) {
                    if (count&1) *in*=n;
                    *out*=sqrt(pow(n,(mpl_real)(count&0xFE)));
								  }

								  if (real==1.0) {
                    if (*in==1.0) *in=0;
                    if (*out==1.0 && *in!=0) *out=0;
                    return(true);
                  }
        }
        }
return(false);
}



bool real_to_rational(mpl_real real,mpl_rational *rat,mpl_rational_info *info,bool use_info,bool test)
{
int i,id;
mpl_real n,up=0,lo=0,pk=real;
bool pass=false;

if (real==0) return(false);

id=0;

while (!pass) {

for (i=0;i<21;i++)
     if (mpl_modul(per_table[i],real)) {
         up=per_table[i];
         lo=round(up/real);
         pass=true;
         break;
     }


if (!pass) {
     if (!test || id==5) return(false);
     if (id==0) real=pk/PI;
     if (id==1) real=pk*PI;
     if (id==2) real=pk*pk;
     if (id==3) real=pk/PI,real=real*real;
     if (id==4) real=pk*PI,real=real*real;
     id++;
}

} // end while


if (info) {
  info->root=false;
  info->pii_location=0;
  if (id==1) info->pii_location=1;
  if (id==2) info->pii_location=2;
  if (id==3) info->root=true;
  if (id==4) info->pii_location=1,info->root=true;
  if (id==5) info->pii_location=2,info->root=true;
}




if (table_of_numbers && count_of_numbers>70)
for(i=0;i<70;i++) {
    n=table_of_numbers[i];
    while (fmod(up,n)==0 && fmod(lo,n)==0) {
		up/=n; lo/=n;								
		}
    if (n>up && n>lo) break;
}
else ErrorReport("Mathematics","No table");

rat->a=up; rat->b=lo;
return(true);
}







